#ifdef LINUX
#include <stdio.h>
#include <errno.h>
#else
#include <windows.h>
#endif /* LINUX */

#include "errmsgs.h"
#include "string.h"

#define MAX_ERR_MSG_LEN	512
#define NO_ERROR_MSG	"No error"

static char last_err_msg[MAX_ERR_MSG_LEN] = NO_ERROR_MSG;
static int last_err_no = 0;
static int last_sys_err_no = 0;

err_struct errors[] =
{
	{ 0, NO_ERROR_MSG },
	{ SER_ERR_TOUT, "Set CommTimeout error" },
	{ SER_ERR_TOUT_NOT_OPEN, "Set Timeout error: Port is not open" },
	{ SER_ERR_CONFIG, "Config error" },
	{ SER_ERR_CONFIG_NOT_OPEN, "Config error: Port is not open" },
	{ SER_ERR_COMSTATE, "Set CommState error" },
	{ SER_ERR_ALREADY_OPEN, "Port is already open" },
	{ SER_ERR_CANT_OPEN, "Open error" },
	{ SER_ERR_OPEN_NO_CHAR_DEV, "Open error: Device is not a character device" },
	{ SER_ERR_CLOSE_NOT_OPEN, "Close error: Device was not open" },
	{ SER_ERR_CANT_CLOSE, "Close error: Unable to close serial port" },
	{ SER_ERR_TX_NOT_OPEN, "Transmit error: Port is not open" },
	{ SER_ERR_TX, "Transmit error" },
	{ SER_ERR_TX_SIZE, "Transmit error: Could not send data" },
   { SER_ERR_TX_TIMEOUT, "Transmit error: TX timeout" },
	{ SER_ERR_RX_NOT_OPEN, "Receive error: Port is not open" },
	{ SER_ERR_RX, "Read error" },
	{ SER_ERR_RX_NO_DATA, "No data to read" },
	{ SER_ERR_NOT_SERIAL, "Open command port is not a serial device" },
	{ SER_INVALID_NAME, "Invalid serial port name specified" },
	{ CMD_ERR_CANT_ERASE, "Failed to erase Flash memory page" },
	{ CMD_ERR_CANT_SET_SUB_PAGE, "Failed to set Flash memory sub-page" },
	{ CMD_ERR_CANT_ERASE_IDX, "Failed to erase NAND flash index" },
	{ CMD_ERR_NSUP_SW_BAUD_RATE, "Switching upload baud rate is not supported on this chip" },
	{ CMD_ERR_CANT_SW_BAUD_RATE, "Failed to switch baud rate" },
	{ CMD_ERR_HAS_UPLD_BAUD_RATE, "Caution: iLCD device has now upload baudrate" },
	{ CMD_ERR_CANT_WRITE, "Failed to write data to Flash" },
	{ CMD_ERR_CANT_READ_FILE, "Failed to read Flash data from file" },
	{ CMD_ERR_REACHED_EOF, "Found unexpected EOF in data file" },
	{ CMD_ERR_CANT_OPEN_FILE, "Failed to open Flash data file" },
	{ CMD_ERR_FILESIZE, "Failed to get Flash data filesize" },
	{ CMD_ERR_NOT_ILCD, "Selected file is not an iLCD Flash data file" },
	{ CMD_ERR_ALREADY_OPEN, "Command port is already open" },
	{ CMD_ERR_ALREADY_CLOSED, "Command port is already closed" },
	{ CMD_ERR_PORT_NOT_OPEN, "Command port not open" },
	{ CMD_ERR_FILE_TOO_BIG, "Data size is too big for the iLCD controller's Flash memory" },
	{ CMD_ERR_CANT_GET_CNAME, "Unable to get the name of the iLCD controller" },
	{ CMD_ERR_INVAL_CHKSUM, "Data file has invalid checksum" },
	{ CMD_ERR_IVAL_CONTR, "Selected file can't be used with the current iLCD controller" },
	{ CMD_ERR_USER_ABORT, "Aborted by user" },    

	{ USB_ERR_DLL_LOAD, "Failed to load DLL for USB" },
	{ USB_ERR_DLL_ENTRY_POINT, "Couldn't find one or more entry points in DLL" },
	{ USB_ERR_ENUM, "USB enumeration error" },
	{ USB_ERR_ALREADY_OPEN, "USB device is already open" },
	{ USB_ERR_WR_CREATE_FILE, "Unable to open USB device" },
	{ USB_ERR_NOT_OPEN, "USB device is not open" },
	{ USB_ERR_WRITE_TIMEOUT, "Encountered timeout on USB device" },
	{ USB_ERR_WRITE, "Error writing data to USB device" },
	{ USB_ERR_PARTIAL_WRITE, "Error writing complete data USB device" },
	{ USB_ERR_CONNECTION_LOST, "USB connection lost" },
	{ USB_ERR_RX_NO_DATA, "No data received on USB device" },
	{ USB_ERR_ENUM_DEV_SET, "Error enumbering USB devices" },
	{ USB_ERR_NO_DEVICES_FOUND, "No iLCD USB devices found" },
   { USB_ERR_LIBUSB_INIT, "Failed to initialize libusb-1.0" },
	{ USB_RD_THREAD_CREATE_ERROR, "Unable to create USB read thread" },
	{ USB_RD_CREATE_FILE_ERR, "Unable to open USB device in read thread" },
	{ ETH_ERR_WSA_LOAD, "Unable to load Winsock" },
	{ ETH_ERR_CREATE_UDP, "Unable to create UDP Socket"},
	{ ETH_ERR_CREATE_TCP, "Unable to create TCP Socket"},
	{ ETH_ERR_SET_SOCK_OPTION, "Unable to set socket options"},
	{ ETH_ERR_SEND_UDP_BROADCAST, "Failed to send UDP broadcast"},
	{ ETH_ERR_UDP_BIND, "Unable to bind udp port"},
	{ ETH_ERR_UDP_RESPONSE, "Got no valid UDP response from device"},
	{ ETH_ERR_RECEIVE_TIMEOUT, "Got no repsone from device (Timeout)"},
	{ ETH_ERR_WRONG_PW, "Entered wrong password"},
	{ ETH_ERR_RECEIVE, "Failed to receive data from device"},
	{ ETH_ERR_NO_DEVICES_FOUND, "Found no iLCD devices over Ethernet"},
	{ ETH_ERR_TCP_CONNECT, "Failed to connect to device"},
	{ ETH_ERR_NOT_CONNECTED, "Device is not connected"},
	{ ETH_ERR_MORE_DEVICES_FOUND, "Found more than one device, please disconnect the others"},
	{ ETH_ERR_NO_PW, "No password specified"	},
   { ETH_ALREADY_CONNECTED, "Device is already connected with another application or PC."},

	{ -1, ""}	// marks end
};

//---------------------------------------------------------------------------
bool Error_SetErr(int error_no)
{
	int i;
	bool found = false;

	for (i = 0; errors[i].err_no >= 0; i++)
	{
		if (errors[i].err_no == error_no)
		{

			strncpy(last_err_msg, errors[i].errmsg, MAX_ERR_MSG_LEN);
			last_err_msg[MAX_ERR_MSG_LEN - 1] = '\0';

			if (error_no & HAS_SYS_ERRNO || error_no & HAS_WSA_ERRNO)
			{
#ifndef LINUX
				LPVOID lpMsgBuf;

            if (error_no & HAS_SYS_ERRNO)
               last_sys_err_no = GetLastError();
            else if (error_no & HAS_WSA_ERRNO)
            	last_sys_err_no = WSAGetLastError();

				FormatMessage(
					FORMAT_MESSAGE_ALLOCATE_BUFFER |
					FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL, last_sys_err_no,
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					(LPTSTR) &lpMsgBuf, 0,  NULL);

            strncat(last_err_msg, "\n", 2);
				strncat(last_err_msg, lpMsgBuf, MAX_ERR_MSG_LEN);
				last_err_msg[MAX_ERR_MSG_LEN] = '\0';
#else
				snprintf(last_err_msg, sizeof(last_err_msg), "%s\n%s",
				         errors[i].errmsg, strerror(errno));
#endif /* LINUX */
			}
			found = true;
		}
	}

	if (!found)
	{
		strncpy(last_err_msg, "Unknown error", MAX_ERR_MSG_LEN);
		last_err_msg[MAX_ERR_MSG_LEN-1] = '\0';
	}

	last_err_no = error_no;

	return false;
}
//---------------------------------------------------------------------------
int Error_GetLastErrNo()
{
	return last_err_no;
}
//---------------------------------------------------------------------------
char *Error_GetLastErrMsg()
{
	return last_err_msg;
}
//---------------------------------------------------------------------------
int Error_GetLastSysErrNo()
{
	if (last_err_no & HAS_SYS_ERRNO)
		return last_sys_err_no;

	return 0;
}
//---------------------------------------------------------------------------


