//---------------------------------------------------------------------------
#include <stdio.h>

#ifndef LINUX
	#include <conio.h>
#endif /* ! LINUX */

#ifdef __BORLANDC__
	#if __BORLANDC__ >= 0x0630
		#define CPP_BUILDER_XE		1
	#endif
#endif

#ifdef CPP_BUILDER_XE
	#include <tchar.h>
#endif	

#include <stdlib.h>
#include <string.h>
#include "commands.h"
#include "errmsgs.h"
#include "ethernet.h"

#define CID "\xAA"
#define MAX_IN_LEN	128
//---------------------------------------------------------------------------

#ifndef LINUX
	#pragma argsused
#endif /* ! LINUX */

#define VERSION					"V5.08"
#define SHOW_SW_VERSION_INFO	true
#define SHOW_FW_VERSION_INFO	true
#define PRESS_KEY_TO_EXIT		true
//---------------------------------------------------------------------------
void exit_func(void)
{
#ifndef LINUX
	printf("\nPress any key to exit program");
	getch();
#endif /* LINUX */
}
//---------------------------------------------------------------------------
#ifdef CPP_BUILDER_XE
int _tmain(int argc, _TCHAR* argv[])
#else 
int main(int argc, char* argv[])
#endif
{
	bool was_terminal_mode;
	char *file_name = NULL;
	char *serial_port = NULL;
	int i;
	bool no_cts = false;
	bool invalid_option = false;
	dword cmd_baud_rate = 115200;
	dword upl_baud_rate = 0;
	char *in_data;

	for (i = 1; i < argc; i++)
	{
		if (*argv[i] != '-')
			file_name = argv[i];
		else
		{
			if (strcmp(argv[i], "-nocts") == 0)
				no_cts = true;
			else if (strncmp(argv[i], "-p", 2) == 0)
				serial_port = argv[i] + 2;
			else if (strncmp(argv[i], "-b", 2) == 0)
				cmd_baud_rate = atol(argv[i] + 2);
			else if (strcmp(argv[i], "-usb") == 0)
				SetCommport(cp_usb);
			else if (strcmp(argv[i], "-eth") == 0)
         {
				SetCommport(cp_ethernet);
				Eth_Init();
        	}
			else if (strncmp(argv[i], "-u", 2) == 0)
				upl_baud_rate = atol(argv[i] + 2);
         else if (strncmp(argv[i], "-w", 2) == 0)
			{
				if (!Eth_setPassword(argv[i] + 2))
            {
					invalid_option = true;
            }
         }
			else
				invalid_option = true;
		}
	}

#if PRESS_KEY_TO_EXIT
	atexit(exit_func);
#endif

	if (invalid_option || (serial_port == NULL && GetCommport() == cp_serial  ) ||
		file_name == NULL || cmd_baud_rate == 0)
	{
		printf("\niLoader %s - Loads raw flash data files into the iLCD device\n", VERSION);
		printf("Copyright (c) demmel products gmbh 2014-2023\n\n");
		printf("Usage:  iLoader filename {-pserial_port|-usb|-eth} [-bbaudrate] [-ubaudrate] [-nocts] [-wpassword]\n\n");
		printf("        -pserial_port Select serial port with name 'serial_port'\n");
		printf("        -bbaudrate    Set command baudrate to 'baudrate'\n");
		printf("                      (300..115200 baud, default 115200 baud)\n");
		printf("        -ubaudrate    Set upload baudrate to 'baudrate'\n");
		printf("                      (300..460800 baud, default is command baudrate)\n");
		printf("        -nocts        Ignore CTS pin of serial port for sending\n");
		printf("        -usb          Use USB port instead of serial port\n");
		printf("        -eth          Use Ethernet instead of serial port\n");
		printf("        -wpassword    Specify password for the device. (max length = 16)\n");
		printf("                      This option only has an effect if Ethernet is used \n");
		printf("\n");
		printf("Note:   Select the file type 'Flash Raw Data (*.lcdp-rflash)' or\n");
		printf("        'Raw iLCD Application Data (*.rid)' on the File->Export\n");
		printf("        screen of the iLCD Manager XE program to create raw flash\n");
		printf("        files suitable for iLoader.\n");
		printf("\n");
		printf("CAVEAT: Ensure the iLCD device is not being connected to some other\n");
		printf("        software (like iLCD Manager) while working with this software.\n\n");
		return 1;
	}

#if SHOW_SW_VERSION_INFO
	printf("\niLoader %s - Loads raw flash data files into the iLCD device\n", VERSION);
	printf("Copyright (c) demmel products gmbh 2014-2023\n\n");
#endif

	if (GetCommport() == cp_usb || GetCommport() == cp_winusb || 
		GetCommport() == cp_ethernet)
	{		
		cmd_baud_rate = upl_baud_rate = 0;
   }

//	comm_open(USB, SERIAL, ETHERNET);
//	comm_write();
//	comm_read();
//	comm_close(USB, SERIAL, ETHERNET);

	if (!Commands_PortOpen(serial_port, cmd_baud_rate, no_cts, commport))
	{
		printf("%s\n", Error_GetLastErrMsg());
		return 1;
	}

	if (!Commands_GetAttention(&was_terminal_mode))
	{
		printf("No or invalid response from iLCD device\n");
		Commands_PortClose();
		return 1;
	}

#if SHOW_FW_VERSION_INFO
	if ((in_data = Commands_GetInfo()) == NULL)
	{
		Commands_PortClose();
		printf("%s\n", Error_GetLastErrMsg());
		return 1;
	}

	// output version info
	printf("%s\n", in_data);
#endif

	if (!Commands_WriteAppData(file_name, cmd_baud_rate, upl_baud_rate))
	{
		Commands_PortClose();
		printf( "%s\n", Error_GetLastErrMsg());
		return 1;
	}

	if (!Commands_Reboot())
	{
		Commands_PortClose();
		printf("Unable to reboot the iLCD device\n");
		return 1;
	}

	Commands_PortClose();
   if (GetCommport() == cp_ethernet)
   {
    	Eth_End();
   }
	return 0;
}
//---------------------------------------------------------------------------

